from __future__ import annotations

import shlex
import subprocess


def run(command: str, with_console: bool = True, line_limit: int | None = None) -> None:
    command_parts = shlex.split(command)
    process = subprocess.run(command_parts, capture_output=True, text=True)
    print()
    if with_console:
        print("```console")
        print(f"$ {command}")

    output = process.stdout.strip()
    if line_limit:
        output = "".join(output.splitlines(keepends=True)[:line_limit]) + "..."
    print(output)

    if with_console:
        print("```")
    print()



"""
linkotron: CLI to format GitHub links in a shorter format.
"""

from __future__ import annotations

import argparse

from . import __version__, shorten

try:
    import pyperclip as copier  # type: ignore[import-untyped]
except ImportError:
    try:
        import xerox as copier  # type: ignore[import-not-found]
    except ImportError:
        copier = None


def main() -> None:
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter
    )
    parser.add_argument(
        "-V", "--version", action="version", version=f"%(prog)s {__version__}"
    )
    parser.add_argument("input", help="text containing GitHub links to shorten")
    parser.add_argument(
        "--no-copy", action="store_true", help="do not copy output to clipboard"
    )

    format_group = parser.add_argument_group("formatters")
    format_group = format_group.add_mutually_exclusive_group()
    for name, help_text in (
        ("md", "Markdown"),
        ("rst", "reStructuredText"),
    ):
        format_group.add_argument(
            f"-{name[0]}",
            f"--{name}",
            f"--{help_text.lower()}",
            action="store_const",
            const=name,
            dest="formatter",
            help=f"output in {help_text}",
        )

    args = parser.parse_args()

    output = shorten(line=args.input, formatter=args.formatter)
    if copier and not args.no_copy and output != args.input:
        copier.copy(output)
        print(f"Copied! {output}")
    else:
        print(f"{output}")


if __name__ == "__main__":
    main()



"""
Unit tests
"""

from __future__ import annotations

import pytest

import linkotron


@pytest.mark.parametrize(
    "link, expected",
    [
        ("https://github.com/python/peps", "python/peps"),
        ("https://github.com/python/peps/issues/2", "python/peps#2"),
        ("https://github.com/python/peps/pull/2399", "python/peps#2399"),
        (
            "https://github.com/"
            "python/peps/commit/ceb81fd7b71f74aaa2295594f86501acbe620cda",
            "python/peps#ceb81fd",
        ),
        (
            "https://github.com/python/peps/pull/2399#issuecomment-1063409480",
            "python/peps#2399 (comment)",
        ),
    ],
)
def test_shorten(link: str, expected: str) -> None:
    # Act / Assert
    assert linkotron.shorten(link) == expected
    assert linkotron.shorten(link + "/") == expected


@pytest.mark.parametrize("link, expected", [("some text", "some text")])
def test_shorten_no_link(link: str, expected: str) -> None:
    # Act / Assert
    assert linkotron.shorten(link) == expected


@pytest.mark.parametrize(
    "link, formatter, expected",
    [
        (
            "https://github.com/python/peps/pull/2399",
            "md",
            "[python/peps#2399](https://github.com/python/peps/pull/2399)",
        ),
        (
            "https://github.com/python/peps/pull/2399",
            "rst",
            "`python/peps#2399 <https://github.com/python/peps/pull/2399>`__",
        ),
    ],
)
def test_shorten_into_format(link: str, formatter: str, expected: str) -> None:
    # Act / Assert
    assert linkotron.shorten(link, formatter=formatter) == expected



"""
CLI to format links
"""

from __future__ import annotations

import re
from typing import Any

from ._version import __version__ as __version__


# https://github.com/nedbat/adventofcode2022/blob/main/day07.py
class RegexMatcher:
    def __init__(self, text: str) -> None:
        self.text = text
        self.m: Any = None

    def __eq__(self, pattern: object) -> bool:
        if not isinstance(pattern, re.Pattern):
            return NotImplemented
        self.m = re.fullmatch(pattern, self.text)
        return bool(self.m)

    def __getitem__(self, group: int | str) -> str:
        return str(self.m[group])


class Patterns:
    # https://stackoverflow.com/a/59082561/724176
    USERNAME = "[a-zA-Z0-9]+([-_][a-zA-Z0-9]+)*"
    REPO = r"[a-zA-Z0-9]+([-_\.][a-zA-Z0-9]+)*[-_\.]?[a-zA-Z0-9]+"

    REPO_URL = re.compile(
        rf"^https://github.com/(?P<username>{USERNAME})/(?P<repo>{REPO})/?$"
    )
    PR_OR_ISSUE = re.compile(
        rf"^https://github.com/(?P<username>{USERNAME})/(?P<repo>{REPO})/"
        r"(pull|issues)/(?P<number>\d+)/?$"
    )
    COMMIT = re.compile(
        rf"^https://github.com/(?P<username>{USERNAME})/(?P<repo>{REPO})/"
        r"commit/(?P<sha>[0-9a-f]+)/?$"
    )
    COMMENT = re.compile(
        rf"^https://github.com/(?P<username>{USERNAME})/(?P<repo>{REPO})/"
        r"(pull|issues)/(?P<number>\d+)#issuecomment-\d+/?$"
    )


def shorten(line: str, *, formatter: str | None = None) -> str:
    """Shorten GitHub links"""
    match m := RegexMatcher(line):
        case Patterns.REPO_URL:
            short = f"{m['username']}/{m['repo']}"
        case Patterns.PR_OR_ISSUE:
            short = f"{m['username']}/{m['repo']}#{m['number']}"
        case Patterns.COMMIT:
            short = f"{m['username']}/{m['repo']}#{m['sha'][:7]}"
        case Patterns.COMMENT:
            short = f"{m['username']}/{m['repo']}#{m['number']} (comment)"
        case _:
            return line

    if formatter in ("md", "markdown"):
        return f"[{short}]({line})"
    elif formatter in ("rst", "restructuredtext"):
        return f"`{short} <{line}>`__"
    return short
